import ListErrors from './ListErrors';
import React from 'react';
import agent from '../agent';
import { connect } from 'react-redux';
import Badwords from './Badwords';
import {
  ADD_TAG,
  EDITOR_PAGE_LOADED,
  REMOVE_TAG,
  ARTICLE_SUBMITTED,
  EDITOR_PAGE_UNLOADED,
  UPDATE_FIELD_EDITOR
} from '../constants/actionTypes';


const mapStateToProps = state => ({
  ...state.editor
});

const mapDispatchToProps = dispatch => ({
  onAddTag: () =>
    dispatch({ type: ADD_TAG }),
  onLoad: payload =>
    dispatch({ type: EDITOR_PAGE_LOADED, payload }),
  onRemoveTag: tag =>
    dispatch({ type: REMOVE_TAG, tag }),
  onSubmit: payload =>
    dispatch({ type: ARTICLE_SUBMITTED, payload }),
  onUnload: payload =>
    dispatch({ type: EDITOR_PAGE_UNLOADED }),
  onUpdateField: (key, value) =>
    dispatch({ type: UPDATE_FIELD_EDITOR, key, value })
});


class Editor extends React.Component {
  constructor() {
    super();

    const updateFieldEvent =
      key => ev => this.props.onUpdateField(key, ev.target.value);
    this.changeTitle = updateFieldEvent('title');
    this.changeDescription = updateFieldEvent('description');
    this.changeBody = updateFieldEvent('body');
    this.changeTagInput = updateFieldEvent('tagInput');

    const errors = (key, error) => this.props.onUpdateField(key, error);

    this.watchForEnter = ev => {
      if (this.props.tagInput < 1 || (this.props.tagList || []).length > 3) {
        errors('errors', {'Tags:': 'Empty tags not allowed'});
        return null
      }
      if (ev.keyCode === 13) {
        ev.preventDefault();
        this.props.onAddTag();
        errors('errors', {});
      }
    };

    this.clickToAdd = ev => {
      if (this.props.tagInput < 1 || (this.props.tagList || []).length > 3) {
        errors('errors', {'Tags:': 'Empty tags not allowed'});
        return null
      } else {
        ev.preventDefault();
        this.props.onAddTag();
        errors('errors', {});
      }
    };

    this.removeTagHandler = tag => () => {
      this.props.onRemoveTag(tag);
    };

    this.submitForm = ev => {
      if (!this.props.title||!this.props.body||!this.props.description) {
        errors('errors', {'Please,': 'fill title and body fields.'});
      } else {
      ev.preventDefault();
      const article = {
        title: this.props.title,
        description: this.props.description,
        body: this.props.body,
        tagList: this.props.tagList
      };
      const slug = { slug: this.props.articleSlug };
      const promise = this.props.articleSlug ?
        agent.Articles.update(Object.assign(article, slug)) :
        agent.Articles.create(article);

      this.props.onSubmit(promise);
    }};
  }

  componentWillReceiveProps(nextProps) {
    if (this.props.match.params.slug !== nextProps.match.params.slug) {
      if (nextProps.match.params.slug) {
        this.props.onUnload();
        return this.props.onLoad(agent.Articles.get(this.props.match.params.slug));
      }
      this.props.onLoad(null);
    }
  }

  componentWillMount() {
    if (this.props.match.params.slug) {
      return this.props.onLoad(agent.Articles.get(this.props.match.params.slug));
    }
    this.props.onLoad(null);
  }

  componentWillUnmount() {
    this.props.onUnload();
  }

  render() {
    return (
      <div className="editor-page">
        <div className="container page">
          <div className="row">
            <div className="col-md-10 offset-md-1 col-xs-12">
        
              <form>
                <fieldset>

                  <fieldset className="form-group">
                    <input
                      className="form-control form-control-lg"
                      type="text"
                      placeholder="Article Title"
                      maxlength="100"
                      value={this.props.title || ""}
                      onChange={this.changeTitle} />
                  </fieldset>

                  <fieldset className="form-group">
                    <input
                      className="form-control"
                      type="text"
                      placeholder="What's this article about?"
                      maxlength="140"
                      value={this.props.description || ""}
                      onChange={this.changeDescription} />
                  </fieldset>

                  <fieldset className="form-group">
                    <textarea
                      className="form-control"
                      rows="8"
                      placeholder="Write your article (in markdown)"
                      maxlength="3000"
                      value={this.props.body || ""}
                      onChange={this.changeBody}>
                    </textarea>
                  </fieldset>

                  <fieldset className="form-group">
                    <input
                      className="form-control"
                      type="text"
                      placeholder="Enter tags"
                      maxlength="15"
                      value={this.props.tagInput ||""}
                      onChange={this.changeTagInput}
                      onKeyUp={this.watchForEnter} 
                      onKeyDown={(e) => {
                        if (e.keyCode === 9) e.preventDefault();
                      }}/>
                    <span onClick={this.clickToAdd} className="tag-enter">Press Enter or Click to Add</span>

                    <div className="tag-list">
                      {
                        (this.props.tagList || []).map(tag => {
                          return (
                            <span className="tag-default tag-pill" key={tag}>
                              <i  className="ion-close-round"
                                  onClick={this.removeTagHandler(tag)}>
                              </i>
                              {tag}
                            </span>
                          );
                        })
                      }
                    </div>
                  </fieldset>

                  <button
                    className="btn btn-lg pull-xs-right btn-danger m-l-1"
                    type="button"
                    onClick={() => this.props.history.goBack()}>
                    Cancel
                  </button>

                  <button
                    className="btn btn-lg pull-xs-right btn-primary"
                    type="button"
                    id="publish"
                    disabled={false}
                    onClick={this.submitForm}>
                    Publish Article
                  </button>

                  <Badwords title={this.props.title}
                            description={this.props.description}
                            body={this.props.body}
                            tags={this.props.tagInput}
                            tagList={this.props.tagList || []}
                            />

                  <ListErrors errors={this.props.errors}></ListErrors>

                </fieldset>
              </form>
            </div>
          </div>
        </div>
      </div>
    );
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(Editor);
